<?php
/*
Plugin Name: Custom Favorites
Plugin URI: http://w-shadow.com/blog/2008/10/31/custom-favorite-actions-for-wp-27-beta/
Description: Lets you choose your own "favorite actions". Beta. 
Version: 0.2
Author: Janis Elsts
Author URI: http://w-shadow.com/blog/
*/

/*
Created by Janis Elsts (email : whiteshadow@w-shadow.com) 
It's GPL.
*/

if (is_admin()) {
	
//Load the "framework"
require 'shadow_plugin_framework.php';

class FavoriteManager extends FavMan_ShadowPluginFramework {
	private $favorites_table = '';
	
  /**
   * FavoriteManager::__construct()
   * Initializes the plugin.
   *
   * @return void
   */
	function __construct(){
		global $wpdb;
		//Set some plugin-specific options
		$this->option_name = 'ws_custom_favorites';
		$this->defaults = array(
		);
		
		$this->magic_hooks = true;
		$this->favorites_table = $wpdb->prefix . "custom_favorites"; //stores user-defined favorites 
		//Call the default constructor
		parent::__construct(__FILE__);
		
		//Hook the "Favorites" dropdown
		add_filter('favorite_actions', array(&$this, 'favorite_actions'), 9001, 1);
		
		//Register my JS includes
		wp_register_script(  
            'favorite_manager_js',  
            WP_CONTENT_URL . '/plugins/' . basename( dirname( __FILE__ ) ) . '/functions.js',
            array( 'jquery' )  
        );  
        wp_enqueue_script('favorite_manager_js');
	}
	
  /**
   * FavoriteManager::activate()
   * Activation hook handler. Creates the DB table.
   *
   * @return void
   */
	function activate(){
		global $wpdb;
		//Run the default hook first
		parent::activate();
		//Create the database table as needed.
		require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		
        $sql="CREATE TABLE ".$this->favorites_table." (
        		user_id BIGINT (15) NOT NULL,
                page VARCHAR(255) NOT NULL DEFAULT '',
                title VARCHAR(255) NOT NULL DEFAULT '',
                UNIQUE KEY k_page (user_id, page)
            );";
        dbDelta($sql);
	}
	
  /**
   * FavoriteManager::hook_admin_print_scripts()
   * Outputs some of plugins scripts and CSS.
   *
   * @return void
   */
	function hook_admin_print_scripts(){
		global $title;
		$current_page = $this->get_current_page();
		$css_url = WP_CONTENT_URL . '/plugins/' . basename( dirname( __FILE__ ) ) . '/style.css';
		?>
<link rel='stylesheet' href='<?php echo $css_url; ?>' type='text/css' />
<script type='text/javascript'>
	var ws_current_page = '<?php echo js_escape($current_page); ?>';
	var ws_page_title = '<?php echo js_escape($title); ?>';
</script>
		<?php
	}
	
  /**
   * FavoriteManager::get_current_page()
   * Get the current URL relative to the Dashboard root. 
   *
   * @return string 
   */
	function get_current_page(){
		$current_page = substr($_SERVER['REQUEST_URI'], strpos($_SERVER['REQUEST_URI'], '/wp-admin/') + strlen('/wp-admin/'));
		if (empty($current_page)) $current_page = 'index.php';
		return $current_page;
	}
	
  /**
   * FavoriteManager::hook_admin_footer()
   * Outputs the "Working..." panel for use in AJAX functions.
   *
   * @return void
   */
	function hook_admin_footer(){
		/*<debugging>
		global $wpdb, $plugin_page, $pagenow, $current_user, $title;
		
		$custom_favorites = $this->get_user_favorites();
		
		//Find the relative URL of this page
		$current_page = $this->get_current_page();
		
		echo '<pre>', print_r($this->options, true);
		print_r($custom_favorites);
		echo "current page = '$current_page'\n";
		echo "title = '$title'\n";
		echo '</pre>';
		//</debugging>
		//*/
		
?>
<div id='ws_loading_panel'>
	<img src='<?php echo WP_CONTENT_URL . '/plugins/' . basename( dirname( __FILE__ ) ) . '/ajax-loader.gif' ?> ' />
	Working...
</div>
<?php		
	}
	
  /**
   * FavoriteManager::get_user_favorites()
   *
   * @param int $user_id The user who's favorites should be retrieved. Default is current user.
   * @return array An array of users favorite pages
   */
	function get_user_favorites($user_id = null){
		global $wpdb, $current_user;
		if ($user_id === null) $user_id = $current_user->id;
		//Get this user's custom favorites
		$sql = $wpdb->prepare(
			"SELECT * FROM $this->favorites_table 
			 WHERE user_id = %d ORDER BY title", 
			 $user_id);
		$favorites = $wpdb->get_results($sql, ARRAY_A);
		return $favorites;
	}
	
  /**
   * FavoriteManager::favorite_actions()
   * Replaces the default favorite actions with a user-defined list.  	
   *
   * @param array $actions
   * @return array 
   */
	function favorite_actions($actions){
		//Get this user's custom favorites
		$custom_favorites = $this->get_user_favorites();
		//Transform the result into the form that WP expects
		$custom_actions = array();
		if (is_array($custom_favorites)) {
			foreach ($custom_favorites as $favorite){
				$custom_actions[$favorite['page']] = array($favorite['title'], 'read'); //note: permissions
			}
		}
		
		//$actions = array_merge($actions, $custom_actions);
		//Replace default values with the users choices
		$actions = $custom_actions;
		
		$actions['javascript:void(0)'] = array('&nbsp;', 'read');
		
		if (isset($actions[$this->get_current_page()]))
			//Add a "Remove" link if currently on a favorited page
			$actions['javascript:removePageFromFavorites()'] = array('Remove this page', 'read');
		else
			//Add an "Add" link if currently on a favorited page 
			$actions['javascript:addPageToFavorites()'] = array('Add this page', 'read');
		return $actions;
	}
	
  /**
   * FavoriteManager::hook_admin_menu()
   * Adds a makeshif "Favorites" menu for older versions of WP. 
   * Caution! : May be a security threat. Implemented for demonstration purposes only. 
   * Please don't use with WP < 2.7.  
   *
   * @return void
   */
	function hook_admin_menu(){
		global $submenu;
		
		//Only add the menu if this version of WP is older than 2.7
		if ($this->is_wp27()) return;
		
		/*
										====    Warning    ==== 
		The below code represents a security threat. It may enable the user to access his favorite'd 
		pages *regardless of* any security settings (user-level/capability) that would have otherwise
		prevented it. It is therefore *not recommended* to use this plugin with WP 2.6.x and older 
		versions. 		  
		*/
		//Add the top menu 
		add_menu_page('Favorites', 'Favorites', 'read', 'dashboard_favorites', array(&$this, 'fake_menu_hook'));
		//Load favorites from DB
		$custom_favorites = $this->get_user_favorites();
		//Add one submenu for each favorite
		$on_favorite_page = false;
		$current_page = $this->get_current_page();
		if (is_array($custom_favorites) && count($custom_favorites)) {
			foreach ($custom_favorites as $favorite){
				add_submenu_page('dashboard_favorites', $favorite['title'], $favorite['title'],
					'read', $favorite['page']);
				if ($favorite['page'] == $current_page) $on_favorite_page = true;
			}
		}
		
		//I condemn WordPress for automatically adding a subitem. We must destroy it, now!
		function adhoc_filter($item) { return ($item[2]!='dashboard_favorites');}
		if (is_array($submenu['dashboard_favorites']))
			$submenu['dashboard_favorites'] = array_filter($submenu['dashboard_favorites'], 'adhoc_filter');
		
		//In fact, do the next bit manually.
		if ($on_favorite_page)
			//Add a "Remove" link if currently on a favorited page
			$submenu['dashboard_favorites'][] = array(
				'Remove this page', 
				'read', 
				'javascript:removePageFromFavorites()',
				'Remove this page',
			);
		else
			//Add an "Add" link if not on a favorited page
			$submenu['dashboard_favorites'][] = array(
				'Add this page', 
				'read', 
				'javascript:addPageToFavorites()',
				'Add this page',
			); 
			
	}
	
  /**
   * FavoriteManager::fake_menu_hook()
   * Automatically redirects the browser back to the last visited page.
   *
   * @return void
   */
	function fake_menu_hook(){
?>
<script type='text/javascript'>
	history.go(-1); 
</script>
There's nothing here!
<?php
	}
	
  /**
   * FavoriteManager::hook_wp_ajax_add_dashboard_favorite()
   * AJAX handler. Records a favorite page for the current user.
   *
   * @return void
   */
	function hook_wp_ajax_add_dashboard_favorite(){
		global $wpdb, $current_user;
		if (!current_user_can('read')){
			die("Error: You are not logged in.");
		}
		$sql = $wpdb->prepare(
			"INSERT INTO $this->favorites_table(user_id, page, title) 
			 VALUES ( %d, %s, %s )
			 ON DUPLICATE KEY UPDATE title = VALUES(title)", 
			 $current_user->ID, $_POST['fav_url'], $_POST['fav_title']);
		$wpdb->query($sql);
		die('OK');
	}
	
  /**
   * FavoriteManager::hook_wp_ajax_remove_dashboard_favorite()
   * AJAX handler. Removes the specified favorite page for the current user.
   *
   * @return void
   */
	function hook_wp_ajax_remove_dashboard_favorite(){
		global $wpdb, $current_user;
		if (!current_user_can('read')){
			die("Error: You are not logged in.");
		}
		$sql = $wpdb->prepare(
			"DELETE FROM $this->favorites_table
			 WHERE user_id = %d AND page = %s", 
			 $current_user->ID, $_POST['fav_url']);
		$wpdb->query($sql);
		if ($wpdb->rows_affected > 0) {
			die('OK');
		} else {
			die("Uh oh. Looks like nothing happened.");
		}
	}
	
  /**
   * FavoriteManager::is_wp27()
   * Checks if running WordPress 2.7 or later.
   *
   * @return boolean
   */
	function is_wp27(){
		global $wp_version;
		return function_exists('register_uninstall_hook');
	}
	
	
	
} //class

$favorite_manager = new FavoriteManager();

} //is_admin()

//Uninstallation support for WP 2.7. A bit inelegant, it is (because you can't really 
//call the plugin's object in this situation). 
if (function_exists('register_uninstall_hook'))
	register_uninstall_hook(__FILE__, 'uninstall_favorite_manager');
function uninstall_favorite_manager(){
	global $wpdb;
	delete_option('ws_custom_favorites');
	$wpdb->query("DROP TABLE ".$wpdb->prefix . "custom_favorites");
}

?>